
.loading-overlay {
  position: fixed;
  inset: 0;
  background: rgba(255, 255, 255, 0.6);
  z-index: 1050;
  display: flex;
  align-items: center;
  justify-content: center;
}

.loading-overlay .spinner-border {
  width: 3rem;
  height: 3rem;
}
