{% extends "index.html" %}

{% block style %}
{% endblock %}

{% block forward_url %}"{{ url_for('index') }}"{% endblock %}

{% block error %}

    <div class="container">
        <h2>Error! Something went wrong!</h2>
        <p style="color: red">
            You will be redirected to the index page in <span id="countdown">5</span> seconds...
        </p>
    </div>

{% endblock %}
